;NSIS Setup Script
;--------------------------------

!define VERSION "1.0"

!define PKG_NAME "UEd TexKit"
!define LONG_PKG_NAME "Texture Toolkit for UnrealEd"
!define INSTPROG_NAME "texkit"

!ifndef OUTFILE
!define OUTFILE "UEdTexKit-${VERSION}.exe"
!endif

;--------------------------------
;Configuration

OutFile "${OUTFILE}"
SetCompressor lzma

InstType "Standard"
InstType "Full"

InstallDir "$PROGRAMFILES\${PKG_NAME}"
InstallDirRegKey HKLM "Software\${PKG_NAME}" ""

;--------------------------------
;Header Files

!include "MUI.nsh"
!include "Sections.nsh"

;--------------------------------
;Definitions

!define SHCNE_ASSOCCHANGED 0x8000000
!define SHCNF_IDLIST 0

;--------------------------------
;Configuration

;Names
Name "${LONG_PKG_NAME}"
Caption "${PKG_NAME} ${VERSION} Setup"

!define MUI_ICON "texkit.ico"
!define MUI_UNICON "texkit.ico"

;Interface Settings
!define MUI_ABORTWARNING

!define MUI_HEADERIMAGE

!define MUI_COMPONENTSPAGE_SMALLDESC

;Pages
!define MUI_WELCOMEPAGE_TITLE "Welcome to the ${PKG_NAME} ${VERSION} Setup Wizard"
!define MUI_WELCOMEPAGE_TEXT "This wizard will guide you through the installation of ${LONG_PKG_NAME}, version ${VERSION}.\r\n\r\n$_CLICK"

!insertmacro MUI_PAGE_WELCOME
Page custom PageReinstall PageLeaveReinstall
!insertmacro MUI_PAGE_COMPONENTS
!insertmacro MUI_PAGE_DIRECTORY
!insertmacro MUI_PAGE_INSTFILES

!define MUI_FINISHPAGE_SHOWREADME "$INSTDIR\README.txt"
!define MUI_FINISHPAGE_NOREBOOTSUPPORT

!insertmacro MUI_PAGE_FINISH

!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES

;--------------------------------
;Languages

!insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Reserve Files

  ;These files should be inserted before other files in the data block

  ReserveFile "${INSTPROG_NAME}.ini"
  !insertmacro MUI_RESERVEFILE_INSTALLOPTIONS

;--------------------------------
;Installer Sections

Section "${PKG_NAME} Utilities (required)" SecUtils

  SetDetailsPrint textonly
  DetailPrint "Installing ${PKG_NAME} Utilities..."
  SetDetailsPrint listonly

  SectionIn 1 2 RO
  SetOutPath $INSTDIR

  SetOverwrite on
  File ..\tgafix.exe
  File ..\tgafix_w.exe
  File ..\g16convert.exe
  File ..\g16convert_w.exe
  File ..\ddsconvert.exe
  File ..\ddsconvert_w.exe
  File ..\README.txt

#  WriteRegStr HKCR "Applications\tgafix_w.exe\shell\open" "FriendlyAppName" "UnrealEd TGA Fixer"

  ReadRegStr $R0 HKCR ".tga" ""
  StrCmp $R0 "" 0 +3
    WriteRegStr HKCR ".tga" "" "TGAImage.Document"
    ReadRegStr $R0 HKCR ".tga" ""
  ReadRegStr $R1 HKCR "$R0" ""
  StrCmp $R1 "" 0 +2
    WriteRegStr HKCR "$R0" "" "TGA Image"
  WriteRegStr HKCR "$R0\shell\tgafix" "" "Fix UnrealEd Export"
  WriteRegStr HKCR "$R0\shell\tgafix\command" "" '"$INSTDIR\tgafix_w.exe" "%1"'

#  WriteRegStr HKCR "Applications\g16convert_w.exe\shell\open" "FriendlyAppName" "G16/TIFF Converter"

  ReadRegStr $R0 HKCR ".bmp" ""
  StrCmp $R0 "" 0 +3
    WriteRegStr HKCR ".bmp" "" "Paint.Picture"
    ReadRegStr $R0 HKCR ".bmp" ""
  ReadRegStr $R1 HKCR "$R0" ""
  StrCmp $R1 "" 0 +2
    WriteRegStr HKCR "$R0" "" "Bitmap Image"
  WriteRegStr HKCR "$R0\shell\g16convert" "" "Convert G16 to TIFF"
  WriteRegStr HKCR "$R0\shell\g16convert\command" "" '"$INSTDIR\g16convert_w.exe" "%1"'

  ReadRegStr $R0 HKCR ".tiff" ""
  StrCmp $R0 "" 0 +3
    WriteRegStr HKCR ".tiff" "" "TIFImage.Document"
    ReadRegStr $R0 HKCR ".tiff" ""
  ReadRegStr $R1 HKCR "$R0" ""
  StrCmp $R1 "" 0 +2
    WriteRegStr HKCR "$R0" "" "TIFF Image"
  WriteRegStr HKCR "$R0\shell\g16convert" "" "Convert to G16"
  WriteRegStr HKCR "$R0\shell\g16convert\command" "" '"$INSTDIR\g16convert_w.exe" "%1"'
  ReadRegStr $R0 HKCR ".tif" ""
  StrCmp $R0 "" 0 +3
    WriteRegStr HKCR ".tif" "" "TIFImage.Document"
    ReadRegStr $R0 HKCR ".tif" ""
  ReadRegStr $R1 HKCR "$R0" ""
  StrCmp $R1 "" 0 +2
    WriteRegStr HKCR "$R0" "" "TIFF Image"
  WriteRegStr HKCR "$R0\shell\g16convert" "" "Convert to G16"
  WriteRegStr HKCR "$R0\shell\g16convert\command" "" '"$INSTDIR\g16convert_w.exe" "%1"'

#  WriteRegStr HKCR "Applications\ddsconvert_w.exe\shell\open" "FriendlyAppName" "DDS to TGA Converter"

  ReadRegStr $R0 HKCR ".dds" ""
  StrCmp $R0 "" 0 +3
    WriteRegStr HKCR ".dds" "" "DDSImage.Document"
    ReadRegStr $R0 HKCR ".dds" ""
  ReadRegStr $R1 HKCR "$R0" ""
  StrCmp $R1 "" 0 +2
    WriteRegStr HKCR "$R0" "" "DirectDraw Surface File"
  WriteRegStr HKCR "$R0\shell\ddsconvert" "" "Convert to TGA"
  WriteRegStr HKCR "$R0\shell\ddsconvert\command" "" '"$INSTDIR\ddsconvert_w.exe" "%1"'

  System::Call 'Shell32::SHChangeNotify(i ${SHCNE_ASSOCCHANGED}, i ${SHCNF_IDLIST}, i 0, i 0)'

SectionEnd

Section "Source Code" SecSource

  SetDetailsPrint textonly
  DetailPrint "Installing Source Code..."
  SetDetailsPrint listonly

  SectionIn 2

  SetOutPath $INSTDIR\src
  File ..\src\Makefile
  File ..\src\tgafix.c
  File ..\src\g16convert.c
  File ..\src\ddsconvert.c
  File ..\src\texkit.rc
  File ..\src\versions.h
  File ..\src\${INSTPROG_NAME}.ico
  File ..\src\${INSTPROG_NAME}.ini
  File ..\src\${INSTPROG_NAME}.nsi
SectionEnd

Section -post

  SetDetailsPrint textonly
  DetailPrint "Creating Registry Keys..."
  SetDetailsPrint listonly

  SetOutPath $INSTDIR

  WriteRegStr HKLM "Software\${PKG_NAME}" "InstallPath" $INSTDIR
  WriteRegStr HKLM "Software\${PKG_NAME}" "Version" "${VERSION}"

  WriteUninstaller "$INSTDIR\uninst-${INSTPROG_NAME}.exe"

  WriteRegExpandStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PKG_NAME}" "UninstallString" '"$INSTDIR\uninst-${INSTPROG_NAME}.exe"'
  WriteRegExpandStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PKG_NAME}" "InstallLocation" "$INSTDIR"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PKG_NAME}" "DisplayName" "${LONG_PKG_NAME}"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PKG_NAME}" "DisplayIcon" "$INSTDIR\uninst-${INSTPROG_NAME}.exe,0"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PKG_NAME}" "DisplayVersion" "${VERSION}"
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PKG_NAME}" "NoModify" "1"
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PKG_NAME}" "NoRepair" "1"

  SetDetailsPrint both

SectionEnd

;--------------------------------
;Descriptions

!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${SecUtils} "Utilities for manipulating texture image files imported and exported from UnrealEd"
  !insertmacro MUI_DESCRIPTION_TEXT ${SecSource} "Source code to the ${PKG_NAME} Utilities and all related files"
!insertmacro MUI_FUNCTION_DESCRIPTION_END

;--------------------------------
;Installer Functions

Function .onInit

  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "${INSTPROG_NAME}.ini"

FunctionEnd

Function PageReinstall

  ReadRegStr $R0 HKLM "Software\${PKG_NAME}" "InstallPath"

  StrCmp $R0 "" 0 +2
    Abort

  ;Detect version
    ReadRegDWORD $R0 HKLM "Software\${PKG_NAME}" "Version"
    StrCmp $R0 ${VERSION} same_version other_version

  other_version:

   !insertmacro MUI_INSTALLOPTIONS_WRITE "${INSTPROG_NAME}.ini" "Field 1" "Text" "A different version of ${PKG_NAME} is installed on your system. It's recommended that you uninstall the current version before installing this one. Select the operation you want to perform and click Next to continue."
   !insertmacro MUI_INSTALLOPTIONS_WRITE "${INSTPROG_NAME}.ini" "Field 2" "Text" "Uninstall before installing"
   !insertmacro MUI_INSTALLOPTIONS_WRITE "${INSTPROG_NAME}.ini" "Field 3" "Text" "Do not uninstall"
   !insertmacro MUI_HEADER_TEXT "Already Installed" "Choose how you want to install ${PKG_NAME}."
   StrCpy $R0 "1"
   Goto reinst_start

  same_version:

   !insertmacro MUI_INSTALLOPTIONS_WRITE "${INSTPROG_NAME}.ini" "Field 1" "Text" "${PKG_NAME} ${VERSION} is already installed. Select the operation you want to perform and click Next to continue."
   !insertmacro MUI_INSTALLOPTIONS_WRITE "${INSTPROG_NAME}.ini" "Field 2" "Text" "Add/Reinstall components"
   !insertmacro MUI_INSTALLOPTIONS_WRITE "${INSTPROG_NAME}.ini" "Field 3" "Text" "Uninstall ${PKG_NAME}"
   !insertmacro MUI_HEADER_TEXT "Already Installed" "Choose the maintenance option to perform."
   StrCpy $R0 "2"

  reinst_start:

  !insertmacro MUI_INSTALLOPTIONS_DISPLAY "${INSTPROG_NAME}.ini"

FunctionEnd

Function PageLeaveReinstall

  !insertmacro MUI_INSTALLOPTIONS_READ $R1 "${INSTPROG_NAME}.ini" "Field 2" "State"

  StrCmp $R0 "1" 0 +2
    StrCmp $R1 "1" reinst_uninstall reinst_done

  StrCmp $R0 "2" 0 +3
    StrCmp $R1 "1" reinst_done reinst_uninstall

  reinst_uninstall:
  ReadRegStr $R1 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PKG_NAME}" "UninstallString"

  ;Run uninstaller
  HideWindow

    ClearErrors
    ExecWait '$R1 _?=$INSTDIR'

    IfErrors no_remove_uninstaller

      Delete $R1
      RMDir $INSTDIR

    no_remove_uninstaller:

  StrCmp $R0 "2" 0 +2
    Quit

  BringToFront

  reinst_done:

FunctionEnd

;--------------------------------
;Uninstaller Section

Section Uninstall

  SetDetailsPrint textonly
  DetailPrint "Uninstalling ${LONG_PKG_NAME}..."
  SetDetailsPrint listonly

  SetDetailsPrint textonly
  DetailPrint "Deleting Registry Keys..."
  SetDetailsPrint listonly

  ReadRegStr $R0 HKCR ".tga" ""
  DeleteRegKey HKCR "$R0\shell\tgafix"

  DeleteRegKey HKCR "TGAImage.Document\shell\tgafix"

  ReadRegStr $R0 HKCR ".bmp" ""
  DeleteRegKey HKCR "$R0\shell\g16convert"

  DeleteRegKey HKCR "Paint.Picture\shell\g16convert"

  ReadRegStr $R0 HKCR ".tiff" ""
  DeleteRegKey HKCR "$R0\shell\g16convert"

  ReadRegStr $R0 HKCR ".tif" ""
  DeleteRegKey HKCR "$R0\shell\g16convert"

  DeleteRegKey HKCR "TIFImage.Document\shell\g16convert"

  ReadRegStr $R0 HKCR ".dds" ""
  DeleteRegKey HKCR "$R0\shell\ddsconvert"

  DeleteRegKey HKCR "DDSImage.Document\shell\ddsconvert"

  DeleteRegKey HKCR "Applications\tgafix_w.exe"
  DeleteRegKey HKCR "Applications\g16convert_w.exe"
  DeleteRegKey HKCR "Applications\ddsconvert_w.exe"

  System::Call 'Shell32::SHChangeNotify(i ${SHCNE_ASSOCCHANGED}, i ${SHCNF_IDLIST}, i 0, i 0)'

  DeleteRegKey HKLM "Software\${PKG_NAME}"

  SetDetailsPrint textonly
  DetailPrint "Deleting Files..."
  SetDetailsPrint listonly

  Delete $INSTDIR\tgafix.exe
  Delete $INSTDIR\tgafix_w.exe
  Delete $INSTDIR\g16convert.exe
  Delete $INSTDIR\g16convert_w.exe
  Delete $INSTDIR\ddsconvert.exe
  Delete $INSTDIR\ddsconvert_w.exe
  Delete $INSTDIR\README.txt
  Delete $INSTDIR\src\Makefile
  Delete $INSTDIR\src\tgafix.c
  Delete $INSTDIR\src\g16convert.c
  Delete $INSTDIR\src\ddsconvert.c
  Delete $INSTDIR\src\texkit.rc
  Delete $INSTDIR\src\versions.h
  Delete $INSTDIR\src\${INSTPROG_NAME}.ico
  Delete $INSTDIR\src\${INSTPROG_NAME}.ini
  Delete $INSTDIR\src\${INSTPROG_NAME}.nsi
  RMDIR $INSTDIR\src

  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PKG_NAME}"
  Delete $INSTDIR\uninst-${INSTPROG_NAME}.exe
  RMDir $INSTDIR

  SetDetailsPrint both

SectionEnd
